"""
.. autoclass:: BandyerClient
   :members:

"""
import os
import pprint
import urllib

import requests


# https://realpython.com/python-send-email/
# https://docs.bandyer.com/Bandyer-RESTAPI/

BASE_URL = "https://sandbox.bandyer.com/rest/"


class BandyerClient:
    """The Bandyer API are organized around REST. Our API has predictable, resource-oriented URLs, and
    use HTTP response codes to indicate API errors. We use built-in HTTP features, like HTTP
    authentication and HTTP verbs, which are understood by off-the-shelf HTTP clients. We support
    cross-origin resource sharing, allowing you to interact securely with our API from a
    client-side web application (though you should never expose your secret API key in any public
    website’s client-side code). JSON is returned by all API responses, including errors.

    """

    #: You can use environment variable ``BANDYER_API_KEY`` or set ``API_KEY`` here
    API_KEY = None

    def __init__(self, log=True, api_key=None):
        """
        Initialize the Bandyer Client class.
        Define 'log' attribute as True if you want to show log messages.

        :param log: bool, if True print logs.
        :param args: extra arguments.
        :param kwargs: extra params.
        """
        self.base_url = BASE_URL
        self.headers = {
            'apikey': api_key or self.API_KEY or os.environ['BANDYER_API_KEY'],
            'Content-Type': 'application/json'
        }
        self.log = log

    def rprint(self, resp):
        """Print response url, status code, reason and json formatted complete response. Define 'log'
        attribute as True during BandyerClient initialization if you want to show log messages. If
        self.log = True: print response url, status code, reason and json formatted complete
        response. If self.log = False: do nothing.

        :param resp: API response.

        """
        if self.log:
            pprint.pprint(resp.url)
            pprint.pprint(resp.status_code)
            pprint.pprint(resp.reason)
            try:
                pprint.pprint(resp.json())
            except BaseException:
                pass
            print("")

    # USER
    def user_list(self):
        """
        Returns a list of user ids.
        If there are no users, the list will be empty. The response will be an array of user ids.

        HTTP Request: GET https://sandbox.bandyer.com/rest/user/list
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-users-list]

        :return: API response (array of user ids).
        """
        endpoint = "user/list"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def user_retrieve(self, user_id):
        """
        Retrieves the details of the user.

        HTTP Request: GET https://sandbox.bandyer.com/rest/user/list/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#retrieve-user]

        :param user_id: The id of the user to retrieve.
        :return: API response (user details)
        """
        endpoint = f"user/list/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def user_create(self, user_id, role="basic", can_video=1, language="it"):
        """
        Create a user in Bandyer. Users are identified by a unique ID which can be customized specifying it as input
        or it will be generated randomly. Each user must have a role. Valid roles are:
        - basic: Allow the user to join rooms.
        - plus: Allow the user to create and join rooms.

        HTTP Request: POST https://sandbox.bandyer.com/rest/user/create
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#create-user]

        :param user_id: Custom user id, useful to avoid the Bandyer/Customer users mapping.
          If specified, it must be unique within your account.
          The custom id can be composed by alphanumeric characters [a-zA-Z] and . _ - symbols.
          If the param is not specified, Bandyer will generate a random ID automatically (Max 128 characters).

        :param role: Roles of the user. Available options are “basic” or “plus”.
        :param can_video: The user’s video publishing permission. Available options are 1 (True) and 0 (False).
        :param language: Language of the user. Available options are “en” or “it”.
        :return: API response (confirmation message).

        """
        if role not in ["basic", "plus"]:
            raise ValueError("Attribute 'role' must be one of the following options: 'basic' or 'plus'")
        if language not in [None, "it", "en"]:
            raise ValueError(
                "Attribute 'language' must be one set to None (Company default language) \
                or one of the following options: 'it' or 'en'"
            )
        endpoint = "user/create"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "id": user_id,
            "role": role,
            "can_video": str(can_video),
            "language": language
        }
        resp = requests.post(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def user_update(self, user_id, can_video=1, language="it"):
        """
        Updates a user by setting the values of the parameters passed as input.
        Not specified parameters will be left unchanged.

        Note: The id and the role cannot be updated.

        HTTP Request:PUT https://sandbox.bandyer.com/rest/user/update/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#update-user]

        :param user_id: The id of the user to update.
        :param can_video: The user’ video publishing permission.
        :param language: Language of the user. Available options are “en” or “it”.
        :return: API response (new user details).
        """
        if language not in [None, "it", "en"]:
            raise ValueError(
                "Attribute 'language' must be one set to None (Company default language) \
                or one of the following options: 'it' or 'en'"
            )
        endpoint = f"user/update/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "can_video": str(can_video),
            "language": language
        }
        resp = requests.put(url, headers=self.headers, json=param_dict)
        self.rprint(resp)
        return resp

    def user_avatar_update(self, user_id, avatar_path):
        """
        Updates a user avatar image uploaded as param in the post request.
        The request must contain the id of the user to update an image to, as the avatar param.
        The provided image should have the 300*300 pixel size, otherwise, Bandyer will automatically resize the
        image for you. The max size permitted to upload is 500Kb per file.
        The request will return the user avatar image URL.

        HTTP Request: PUT https://sandbox.bandyer.com/rest/user/avatar/update/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#update-user-avatar]

        :param user_id: The id of the user to update.
        :param avatar_path: The user avatar image.
          The mime-type must be image/png or image/jpeg. Max file size is 500Kb.
          Suggested dimensions are 300*300 pixels.
        :return: API response (user avatar image URL).
        """
        headers = self.headers
        headers['Content-Type'] = 'multipart/form-data'
        endpoint = f"user/avatar/update/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "avatar": avatar_path,
        }
        resp = requests.put(url, headers=headers, json=param_dict)
        self.rprint(resp)
        return resp

    def user_update_permissions(self, can_video=1):
        """
        Updates video permission to all users of a company.
        The request must contain the can_video permission parameter.
        The request will response with the number of user updated with the value of permissions updated.

        HTTP Request: PUT https://sandbox.bandyer.com/rest/user/update/all/permissions
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#update-all-users-permissions]

        :param can_video: The user’ video publishing permission.
        :return: API response (modification and count).
        """
        endpoint = f"user/update/all/permissions"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "can_video": str(can_video),
        }
        resp = requests.put(url, headers=self.headers, json=param_dict)
        self.rprint(resp)
        return resp

    def user_delete(self, user_id):
        """
        By specifying the id of the user is possible to delete a user from the platform.
        The response will contain a confirmation message.

        HTTP Request: DELETE https://sandbox.bandyer.com/rest/user/delete/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#delete-user]

        :param user_id: The id of the user to delete.
        :return: API response (confirmation message).
        """
        endpoint = f"user/delete/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.delete(url, headers=self.headers, json=param_dict)
        self.rprint(resp)
        return resp

    def user_room_list(self, user_id):
        """
        Get rooms created by a specific user or those he attended.
        If the user does not exist or has no rooms the returned list will be empty.

        The response has a [room_list] key containing an array of rooms in which
        is specified the [room_id] and the [creation_date]

        This REST endpoint has a filter parameter with two possible keys (filter[limit] and filter[skip]).
        This two filter can be used to retrieve a specific number of rooms(filter[limit]) or for example to
        paginate the results through the use of the skip filter (filter[skip]).

        Note: All date fields follow the ISO8601 standard.

        HTTP Request: GET https://sandbox.bandyer.com/rest/user/room/list/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-room-list]

        :param user_id: The id of the user to retrieve room IDs.
        :return: API response (rooms created by a specific user or those he attended)

        """
        endpoint = f"user/room/list/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, json=param_dict)
        self.rprint(resp)
        return resp

    def user_chat_list(self, user_id):
        """
        Returns the list of the chats of the user.
        Each element of the chat_list array contains information about the chat such as,
        the unread message count and the last message sent.

        HTTP Request: GET https://sandbox.bandyer.com/rest/user/:user_id/chat/list
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-chat-list]
        :param user_id: The id of the user to retrieve the chat list.
        :return: API response (list of the chats of the user)
        """
        endpoint = f"user/{user_id}/chat/list"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, json=param_dict)
        self.rprint(resp)
        return resp

    def user_broadcast_list(self, user_id):
        """
        Get broadcasts created by a specific user or those he attended.
        If the user does not exist or has no broadcasts the returned list will be empty.

        The response has a [broadcast_list] key containing an array of rooms in which is specified the [broadcast_id]
        and the [creation_date]

        This REST endpoint has a filter parameter with two possible keys (filter[limit] and filter[skip]).
        This two filter can be used to retrieve a specific number of broadcasts(filter[limit]) or for example
        to paginate the results through the use of the skip filter (filter[skip])

        Note: All date’s fields follow the ISO8601 standard.

        HTTP Request: GET https://sandbox.bandyer.com/rest/user/broadcast/list/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-room-list]

        :param user_id: The id of the user to retrieve broadcast IDs.
        :return: API response (list of broadcast of the user)
        """
        endpoint = f"user/broadcast/list/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, json=param_dict)
        self.rprint(resp)
        return resp

    # CHAT
    def chat_message_list(self, user_list, order="asc", page_number=1, page_size=10, page_token=""):
        """
        Get the chat messages exchanged between the users passed as input.
        The endpoint is paginated because the number of messages can increase indefinitely.
        The response has a messages key containing an array of messages and a meta key that contains
        metadata to use to implement pagination.
        More specifically, there are a next and a previous keys containing the parameters
        to pass to obtain respectively the next and the previous page of messages.

        HTTP Request: GET https://sandbox.bandyer.com/rest/chat/messages
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-chat-messages]

        :param user_list: Array of valid user aliases.
        :param order: The sort order of the returned messages [asc, desc]. Default: order = "asc".
        :param page_number: This number is just a counter to keep track of the page number requested on the client.
        :param page_size: Max number of messages returned in a single request.
        :param page_token: A token provided in each response to keep track of the page index requested so far.

        Include the token in the request obtain the subsequent page.

        :return: API response (paginated array of massages).
        """
        if order not in ["asc", "desc"]:
            raise ValueError(
                "Attribute 'order' must be one set to one of the following options: 'asc' or 'desc'"
            )
        endpoint = "chat/messages"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "users": user_list,
            "order": order,
            "page_number": str(page_number),
            "page_size": str(page_size),
            "page_token": page_token
        }
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def add_chat(self, user_list):
        """
        Create a chat between the users passed as input.
        The response, specify the users passed in input,
        this means that the chat is created as reported in the message field with the “Chat_added” message.

        HTTP Request: POST https://sandbox.bandyer.com/rest/chat/add
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#add-chat]

        :param user_list: Array of valid user aliases.
        :return: API response (confirmation with users passed in input).
        """
        endpoint = "chat/add"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "users": user_list
        }
        resp = requests.post(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def remove_chat(self, user_list):
        """
        Remove a chat between the users passed as input.
        The response, specify the users passed in input,
        this means that the chat is removed as reported in the message field with the “Chat_removed” message.

        HTTP Request: POST https://sandbox.bandyer.com/rest/chat/remove
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#remove-chat]

        :param user_list: Array of valid user aliases.
        :return: API response (confirmation with users passed in input).
        """
        endpoint = "chat/remove"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "users": user_list
        }
        resp = requests.post(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def chat_send_message(self, sender, user_list, message):
        """
        Send a message from a given user, in a specified channel.
        The response, specify the users passed in input, the sender who send the message,
        in addition to the text of the message there is the index and the timestamp.

        HTTP Request: POST https://sandbox.bandyer.com/rest/chat/add
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#send-message]

        :param sender: User who sends the message.
        :param user_list: Array of valid user aliases.
        :param message: Text of the message.
        :return: API response (confirmation with users passed in input).
        """
        endpoint = "chat/add"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "sender": sender,
            "users": user_list,
            "message": message,
        }
        resp = requests.post(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    # ROOM
    def room_create(
            self, initiator, user_list, duration, description, can_record=0,
            call_type="audio_video", direct_call=0, live=0, initiator_alias=""
    ):
        """
        Create a room.
        The duration parameter indicates the maximum duration of the call. It is expressed in seconds.
        If the parameter is not specified, the call has unlimited duration. The minimum duration is 60 minutes.
        The directCall parameter indicates the type of link provided by the endpoint.
        If directCall is true, the user will have direct access to the call.
        The user will not see the Bandyer landing page with information about invited users and call options.
        Default is false.
        The response returns the URLs that each user can use to join the call.

        Note: Within the sandbox server, calls won’t be registered even if indicated in the REST parameters.
        If you would like to try the recording option in sandbox, please contact us at support [support@bandyer.com].

        Note: 30 seconds before the time is up all clients get notifications of the short time left.
        When the duration expires the call ends. Available time runs only when clients are in the videocall.
        If they are experiencing problems and the connection drops, the timer will stop.

        Note: If you have enabled WebHooks (described in Web Hooks section
        [https://docs.bandyer.com/Bandyer-RESTAPI/#webhooks]), the room_id returned in the response,
        could be saved in your database for further hook events that might be received.

        HTTP Request: POST https://sandbox.bandyer.com/rest/room/create
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#create-a-room]

        :param initiator: Room creator user id.
        :param user_list: The array of the user alias that will participate to the room.
        :param duration: Maximum value expressed in seconds of effective calling time
            allowed for the room.

            The minimum value is 60. If not specified, the call duration is unlimited.
            With effective calling time is intended the sum of the time where the room is running with
            at least two connected users.

        :param description: A textual description of the room.
        :param can_record: This param defines whether the room should be recorded “1” or not “0”.
        :param call_type: This is the call type, it can be ‘audio_only’, 'audio_upgradable’
          or 'audio_video’ as default.
        :param direct_call: Set this boolean if you want direct access in the call.
        :param live: This param defines whether the link should trigger widget and mobile ringing or not.
           If true it also involves the firing of the on_call_incoming webhook.
        :param initiator_alias: Room creator alias (a valid user alias).
        :return: API response (URLs that each user can use to join the call).
        """
        if call_type not in ["audio_video", "only_audio", "audio_upgradable"]:
            raise ValueError(
                "Attribute 'call_type' must be one set to one of the following options: \
                'audio_video', 'only_audio' or 'audio_upgradable'"
            )
        if str(can_record) not in ["0", "1"]:
            raise ValueError(
                "Attribute 'can_record' must be one set to one of the following options: 0 or 1"
            )
        if str(direct_call) not in ["0", "1"]:
            raise ValueError(
                "Attribute 'direct_call' must be one set to one of the following options: 0 or 1"
            )
        if str(live) not in ["0", "1"]:
            raise ValueError(
                "Attribute 'live' must be one set to one of the following options: 0 or 1"
            )
        endpoint = "room/create"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "initiator": initiator,
            "user_list_in_call": user_list,
            "duration": duration,
            "description": description,
            "record": str(can_record),
            "call_type": call_type,
            "directCall": str(direct_call),
            "live": str(live),
            "initiatorAlias": initiator_alias,
        }
        resp = requests.post(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def room_disable(self, room_id):
        """
        This endpoint disable a specific room and all URL token link associated with it.
        A disabled room will not able to be used again, however, the latter will not be deleted
        from Bandyer’s data structure, all statistics will be still available.
        The request will reply with a message. In case of error, the message will explain the problem.

        HTTP Request: PUT https://sandbox.bandyer.com/rest/room/disable/:room_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#disable-room]

        :param room_id: The id of the room to disable.
        :return: API response (confirmation or error message).
        """
        endpoint = f"room/disable/{room_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.put(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def room_delete(self, room_id):
        """
        This endpoint deletes a specific room and all URL token link associated with it.
        The request will reply with a message. In case of error, the message will explain the problem.

        HTTP Request: DELETE https://sandbox.bandyer.com/rest/room/delete/:room_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#disable-room]

        :param room_id:  The id of the room to delete.
        :return: API response (confirmation or error message).
        """
        endpoint = f"room/delete/{room_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.delete(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def room_record_retrieve(self, session_id, expire_time=10000):
        """
        This endpoint is used to obtain a session recording link.
        In order to get a recording session URL, you have to choose the sessionID from Get Room Information section.

        Note: In the Get Room Information section, you can find all sessions’ information belonging
        to that specific room. To find the sessionID needed, you can set up your favorite query
        such as: duration/creation_date/participants choosing among the key set in the response object.

        Note: Our servers take some time to process the recorded sessions,
        in case your registration is not yet available, this rest will return an error.

        HTTP Request: GET https://sandbox.bandyer.com/rest/room/recording/:session_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-sessions-recording]

        :param session_id: The id of a room session.
        :param expire_time: The quantity of validity time [ms] of the URL generated.
        :return: API response.
        """
        endpoint = f"room/recording/{session_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "expire_time": str(expire_time),
        }
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def room_information_retrieve(self, room_id):
        """
        This end-point could be used to get room information through a valid room_id.

        HTTP Request: GET https://sandbox.bandyer.com/rest/room/info/:room_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-room-information]

        :param room_id: The id of the room that you want the informations.
        :return: API response (One of the parameters in the response is status,
          this parameter explains the status of the room as you can see in the below:
          - Status = "NOT_RUNNING" --> Current room is not running;
          - Status = "RUNNING" --> Current room is running;
          - Status = "CLOSED" --> Current room is deleted or ended (not avaiable).

          Another particular parameter is 'durationUsed' that explains for how many seconds the room was in RUNNING state.
          If the duration used is equals to duration parameter, the room status will be CLOSED.
          The 'dialInfo' section contains informations about sessions and users: room total time and users,
          detail for each session (with partial time and user count) and every user activity and event.)
        """
        endpoint = f"room/info/{room_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    # BROADCAST
    def broadcast_list(self, skip=None, limit=None):
        """
        This endpoint returns the list of broadcasts created for the company matching the provided apikey.

        Note: All date’s fields follow the ISO8601 standard.

        HTTP Request: GET https://sandbox.bandyer.com/rest/broadcast/list
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-broadcast-list]

        :param skip: Skips the first the specified number of returned records.
        :param limit: Limits the number of returned records to the specified number(or less if there aren’t enough).
        :return: API response (list of broadcasts and their details)
        """
        endpoint = "broadcast/list"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "skip": skip,
            "limit": limit,
        }
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def broadcast_retrieve(self, broadcast_id):
        """
        This end-point could be used to get broadcast information through a valid broadcast_id.

        Note: All date’s fields follow the ISO8601 standard.

        HTTP Request: GET https://sandbox.bandyer.com/rest/broadcast/info/:broadcast_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-broadcast-informations]

        :param broadcast_id: The id of the broadcast that you want the informations.
        :return: API response (One of the parameters in the response is status,
           this parameter explains the status of the broadcast as you can see in the below:

           - Status = "NOT_RUNNING" --> Current broadcast is not running;
           - Status = "RUNNING" --> Current broadcast is running;
           - Status = "CLOSED" --> Current broadcast is deleted or ended (not avaiable).

        Another particular parameter is 'totalDurationTime' that explains for how many time the broadcast was in
        RUNNING state.
        The dialInfo section contains informations about all broadcast sessions: room total time,
        detail for each sesssion (with partial time) and every information about subscribers connected for each session.
        And creation the creation date for each session.)
        """
        endpoint = f"broadcast/info/{broadcast_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {}
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def broadcast_create(self, publisher_id, description, can_record=0):
        """
        The broadcast call is composed by one publisher and N subscribers.
        The publisher is the user who will present the event.
        The subscribers are the users who will attend the event.
        The publisher can use his microphone and webcam in order to present the webinar.
        He can also use additional features such as screen-sharing, chat and file sharing.
        This type of call is mainly indicated for hosting an event with a high number of subscribers (~2000).

        The request must contain the id of the user who will be the presenter of the broadcast event.
        It is possible to choose the name of the event and whether the event will be recorded or not.
        The response returns the broadcast id which will be used to join the broadcast event created.

        HTTP Request: POST https://sandbox.bandyer.com/rest/broadcast/create
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#create-broadcast]

        :param publisher_id: Id of the user who will be the publisher of the broadcast event.
        :param description: Name of the event.
        :param can_record: 1 if the user wants to record the videocall, 0 otherwise.
        :return: API response (broadcast_id).
        """
        endpoint = "broadcast/create"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "publisher_id": publisher_id,
            "description": description,
            "record": str(can_record)
        }
        resp = requests.post(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def broadcast_call(self, user_id, broadcast_id, direct=0):
        """To join a broadcast event, the user needs to be redirect in a specific Bandyer page. The
        request must contain the broadcast id of the event and the user id who has requested to
        join it. The response returns the url in which either the publisher or the subscriber must
        be redirected to join the broadcast event.

        HTTP Request: GET https://sandbox.bandyer.com/rest/call/:user_id
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#call-broadcast]

        :param user_id: Id of the user who join the event.
        :param broadcast_id: Broadcast id of the event.
        :param direct: if set to true it will generate a link that will give direct access to the call
           without opening a new popup window.
        :return: API response (url in which either the publisher or the subscriber must be redirected to join
           the broadcast event).

        """
        endpoint = f"call/{user_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "callee": broadcast_id,
            "direct": str(direct)
        }
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp

    def broadcast_record_retrieve(self, session_id, expire_time):
        """
        This endpoint is used to obtain a broadcast session recording link. In order to get a recording session URL,
        you have to choose the sessionID from Get Broadcast Information section.

        Note: In the Get Broadcast Information section, you can find all sessions’ information belonging to
        that specific broadcast. To find the sessionID needed, you can set up your favourite query such as:
        duration/creation_date/participants choosing among the key set in the response object.

        HTTP Request: GET https://sandbox.bandyer.com/rest/broadcast/recording/:session_id?expire_time=30000
        Source: [https://docs.bandyer.com/Bandyer-RESTAPI/#get-sessions-recording29]

        :param session_id: The id of a broadcast session.
        :param expire_time: The quantity of validity time [ms] of the URL generated.
        :return: API response (broadcast record details).

        """
        endpoint = f"recording/{session_id}"
        url = urllib.parse.urljoin(self.base_url, endpoint)
        param_dict = {
            "expire_time": expire_time,
        }
        resp = requests.get(url, headers=self.headers, params=param_dict)
        self.rprint(resp)
        return resp
